@@ -1,16 +1,25 @@
 use crate::ast::{Enum, Field, Input, Struct};
 use crate::attr::Trait;
+use crate::fallback;
 use crate::generics::InferredBounds;
-use proc_macro2::TokenStream;
+use crate::unraw::MemberUnraw;
+use proc_macro2::{Ident, Span, TokenStream};
 use quote::{format_ident, quote, quote_spanned, ToTokens};
 use std::collections::BTreeSet as Set;
-use syn::spanned::Spanned;
-use syn::{
-    Data, DeriveInput, GenericArgument, Member, PathArguments, Result, Token, Type, Visibility,
-};
+use syn::{DeriveInput, GenericArgument, PathArguments, Result, Token, Type};
 
-pub fn derive(node: &DeriveInput) -> Result<TokenStream> {
-    let input = Input::from_syn(node)?;
+pub fn derive(input: &DeriveInput) -> TokenStream {
+    match try_expand(input) {
+        Ok(expanded) => expanded,
+        // If there are invalid attributes in the input, expand to an Error impl
+        // anyway to minimize spurious secondary errors in other code that uses
+        // this type as an Error.
+        Err(error) => fallback::expand(input, error),
+    }
+}
+
+fn try_expand(input: &DeriveInput) -> Result<TokenStream> {
+    let input = Input::from_syn(input)?;
     input.validate()?;
     Ok(match input {
         Input::Struct(input) => impl_struct(input),
@@ -19,94 +28,95 @@ pub fn derive(node: &DeriveInput) -> Result<TokenStream> {
 }
 
 fn impl_struct(input: Struct) -> TokenStream {
-    let ty = &input.ident;
+    let ty = call_site_ident(&input.ident);
     let (impl_generics, ty_generics, where_clause) = input.generics.split_for_impl();
     let mut error_inferred_bounds = InferredBounds::new();
 
-    let source_body = if input.attrs.transparent.is_some() {
+    let source_body = if let Some(transparent_attr) = &input.attrs.transparent {
         let only_field = &input.fields[0];
         if only_field.contains_generic {
-            error_inferred_bounds.insert(only_field.ty, quote!(std::error::Error));
+            error_inferred_bounds.insert(only_field.ty, quote!(::thiserror::__private::Error));
         }
         let member = &only_field.member;
-        Some(quote! {
-            std::error::Error::source(self.#member.as_dyn_error())
+        Some(quote_spanned! {transparent_attr.span=>
+            ::thiserror::__private::Error::source(self.#member.as_dyn_error())
         })
     } else if let Some(source_field) = input.source_field() {
         let source = &source_field.member;
         if source_field.contains_generic {
             let ty = unoptional_type(source_field.ty);
-            error_inferred_bounds.insert(ty, quote!(std::error::Error + 'static));
+            error_inferred_bounds.insert(ty, quote!(::thiserror::__private::Error + 'static));
         }
         let asref = if type_is_option(source_field.ty) {
             Some(quote_spanned!(source.span()=> .as_ref()?))
         } else {
             None
         };
-        let dyn_error = quote_spanned!(source.span()=> self.#source #asref.as_dyn_error());
+        let dyn_error = quote_spanned! {source_field.source_span()=>
+            self.#source #asref.as_dyn_error()
+        };
         Some(quote! {
-            std::option::Option::Some(#dyn_error)
+            ::core::option::Option::Some(#dyn_error)
         })
     } else {
         None
     };
     let source_method = source_body.map(|body| {
         quote! {
-            fn source(&self) -> std::option::Option<&(dyn std::error::Error + 'static)> {
-                use thiserror::__private::AsDynError;
+            fn source(&self) -> ::core::option::Option<&(dyn ::thiserror::__private::Error + 'static)> {
+                use ::thiserror::__private::AsDynError as _;
                 #body
             }
         }
     });
 
     let provide_method = input.backtrace_field().map(|backtrace_field| {
-        let demand = quote!(demand);
+        let request = quote!(request);
         let backtrace = &backtrace_field.member;
         let body = if let Some(source_field) = input.source_field() {
             let source = &source_field.member;
             let source_provide = if type_is_option(source_field.ty) {
                 quote_spanned! {source.span()=>
-                    if let std::option::Option::Some(source) = &self.#source {
-                        source.provide(#demand);
+                    if let ::core::option::Option::Some(source) = &self.#source {
+                        source.thiserror_provide(#request);
                     }
                 }
             } else {
                 quote_spanned! {source.span()=>
-                    self.#source.provide(#demand);
+                    self.#source.thiserror_provide(#request);
                 }
             };
             let self_provide = if source == backtrace {
                 None
             } else if type_is_option(backtrace_field.ty) {
                 Some(quote! {
-                    if let std::option::Option::Some(backtrace) = &self.#backtrace {
-                        #demand.provide_ref::<std::backtrace::Backtrace>(backtrace);
+                    if let ::core::option::Option::Some(backtrace) = &self.#backtrace {
+                        #request.provide_ref::<::thiserror::__private::Backtrace>(backtrace);
                     }
                 })
             } else {
                 Some(quote! {
-                    #demand.provide_ref::<std::backtrace::Backtrace>(&self.#backtrace);
+                    #request.provide_ref::<::thiserror::__private::Backtrace>(&self.#backtrace);
                 })
             };
             quote! {
-                #[allow(unused_imports)]
-                use std::error::Error as _;
+                use ::thiserror::__private::ThiserrorProvide as _;
                 #source_provide
                 #self_provide
             }
         } else if type_is_option(backtrace_field.ty) {
             quote! {
-                if let std::option::Option::Some(backtrace) = &self.#backtrace {
-                    #demand.provide_ref::<std::backtrace::Backtrace>(backtrace);
+                if let ::core::option::Option::Some(backtrace) = &self.#backtrace {
+                    #request.provide_ref::<::thiserror::__private::Backtrace>(backtrace);
                 }
             }
         } else {
             quote! {
-                #demand.provide_ref::<std::backtrace::Backtrace>(&self.#backtrace);
+                #request.provide_ref::<::thiserror::__private::Backtrace>(&self.#backtrace);
             }
         };
         quote! {
-            fn provide<'_demand>(&'_demand self, #demand: &mut std::any::Demand<'_demand>) {
+            fn provide<'_request>(&'_request self, #request: &mut ::core::error::Request<'_request>) {
                 #body
             }
         }
@@ -117,18 +127,11 @@ fn impl_struct(input: Struct) -> TokenStream {
         let only_field = &input.fields[0].member;
         display_implied_bounds.insert((0, Trait::Display));
         Some(quote! {
-            std::fmt::Display::fmt(&self.#only_field, __formatter)
+            ::core::fmt::Display::fmt(&self.#only_field, __formatter)
         })
     } else if let Some(display) = &input.attrs.display {
-        display_implied_bounds = display.implied_bounds.clone();
-        let use_as_display = if display.has_bonus_display {
-            Some(quote! {
-                #[allow(unused_imports)]
-                use thiserror::__private::{DisplayAsDisplay, PathAsDisplay};
-            })
-        } else {
-            None
-        };
+        display_implied_bounds.clone_from(&display.implied_bounds);
+        let use_as_display = use_as_display(display.has_bonus_display);
         let pat = fields_pat(&input.fields);
         Some(quote! {
             #use_as_display
@@ -150,9 +153,10 @@ fn impl_struct(input: Struct) -> TokenStream {
         let display_where_clause = display_inferred_bounds.augment_where_clause(input.generics);
         quote! {
             #[allow(unused_qualifications)]
-            impl #impl_generics std::fmt::Display for #ty #ty_generics #display_where_clause {
+            #[automatically_derived]
+            impl #impl_generics ::core::fmt::Display for #ty #ty_generics #display_where_clause {
                 #[allow(clippy::used_underscore_binding)]
-                fn fmt(&self, __formatter: &mut std::fmt::Formatter) -> std::fmt::Result {
+                fn fmt(&self, __formatter: &mut ::core::fmt::Formatter) -> ::core::fmt::Result {
                     #body
                 }
             }
@@ -160,21 +164,33 @@ fn impl_struct(input: Struct) -> TokenStream {
     });
 
     let from_impl = input.from_field().map(|from_field| {
+        let span = from_field.attrs.from.unwrap().span;
         let backtrace_field = input.distinct_backtrace_field();
         let from = unoptional_type(from_field.ty);
-        let body = from_initializer(from_field, backtrace_field);
-        quote! {
-            #[allow(unused_qualifications)]
-            impl #impl_generics std::convert::From<#from> for #ty #ty_generics #where_clause {
-                #[allow(deprecated)]
-                fn from(source: #from) -> Self {
-                    #ty #body
-                }
+        let source_var = Ident::new("source", span);
+        let body = from_initializer(from_field, backtrace_field, &source_var);
+        let from_function = quote! {
+            fn from(#source_var: #from) -> Self {
+                #ty #body
             }
-        }
+        };
+        let from_impl = quote_spanned! {span=>
+            #[automatically_derived]
+            impl #impl_generics ::core::convert::From<#from> for #ty #ty_generics #where_clause {
+                #from_function
+            }
+        };
+        Some(quote! {
+            #[allow(
+                deprecated,
+                unused_qualifications,
+                clippy::elidable_lifetime_names,
+                clippy::needless_lifetimes,
+            )]
+            #from_impl
+        })
     });
 
-    let error_trait = spanned_error_trait(input.original);
     if input.generics.type_params().next().is_some() {
         let self_token = <Token![Self]>::default();
         error_inferred_bounds.insert(self_token, Trait::Debug);
@@ -184,7 +200,8 @@ fn impl_struct(input: Struct) -> TokenStream {
 
     quote! {
         #[allow(unused_qualifications)]
-        impl #impl_generics #error_trait for #ty #ty_generics #error_where_clause {
+        #[automatically_derived]
+        impl #impl_generics ::thiserror::__private::Error for #ty #ty_generics #error_where_clause {
             #source_method
             #provide_method
         }
@@ -194,20 +211,22 @@ fn impl_struct(input: Struct) -> TokenStream {
 }
 
 fn impl_enum(input: Enum) -> TokenStream {
-    let ty = &input.ident;
+    let ty = call_site_ident(&input.ident);
     let (impl_generics, ty_generics, where_clause) = input.generics.split_for_impl();
     let mut error_inferred_bounds = InferredBounds::new();
 
     let source_method = if input.has_source() {
         let arms = input.variants.iter().map(|variant| {
             let ident = &variant.ident;
-            if variant.attrs.transparent.is_some() {
+            if let Some(transparent_attr) = &variant.attrs.transparent {
                 let only_field = &variant.fields[0];
                 if only_field.contains_generic {
-                    error_inferred_bounds.insert(only_field.ty, quote!(std::error::Error));
+                    error_inferred_bounds.insert(only_field.ty, quote!(::thiserror::__private::Error));
                 }
                 let member = &only_field.member;
-                let source = quote!(std::error::Error::source(transparent.as_dyn_error()));
+                let source = quote_spanned! {transparent_attr.span=>
+                    ::thiserror::__private::Error::source(transparent.as_dyn_error())
+                };
                 quote! {
                     #ty::#ident {#member: transparent} => #source,
                 }
@@ -215,7 +234,7 @@ fn impl_enum(input: Enum) -> TokenStream {
                 let source = &source_field.member;
                 if source_field.contains_generic {
                     let ty = unoptional_type(source_field.ty);
-                    error_inferred_bounds.insert(ty, quote!(std::error::Error + 'static));
+                    error_inferred_bounds.insert(ty, quote!(::thiserror::__private::Error + 'static));
                 }
                 let asref = if type_is_option(source_field.ty) {
                     Some(quote_spanned!(source.span()=> .as_ref()?))
@@ -223,19 +242,21 @@ fn impl_enum(input: Enum) -> TokenStream {
                     None
                 };
                 let varsource = quote!(source);
-                let dyn_error = quote_spanned!(source.span()=> #varsource #asref.as_dyn_error());
+                let dyn_error = quote_spanned! {source_field.source_span()=>
+                    #varsource #asref.as_dyn_error()
+                };
                 quote! {
-                    #ty::#ident {#source: #varsource, ..} => std::option::Option::Some(#dyn_error),
+                    #ty::#ident {#source: #varsource, ..} => ::core::option::Option::Some(#dyn_error),
                 }
             } else {
                 quote! {
-                    #ty::#ident {..} => std::option::Option::None,
+                    #ty::#ident {..} => ::core::option::Option::None,
                 }
             }
         });
         Some(quote! {
-            fn source(&self) -> std::option::Option<&(dyn std::error::Error + 'static)> {
-                use thiserror::__private::AsDynError;
+            fn source(&self) -> ::core::option::Option<&(dyn ::thiserror::__private::Error + 'static)> {
+                use ::thiserror::__private::AsDynError as _;
                 #[allow(deprecated)]
                 match self {
                     #(#arms)*
@@ -247,7 +268,7 @@ fn impl_enum(input: Enum) -> TokenStream {
     };
 
     let provide_method = if input.has_backtrace() {
-        let demand = quote!(demand);
+        let request = quote!(request);
         let arms = input.variants.iter().map(|variant| {
             let ident = &variant.ident;
             match (variant.backtrace_field(), variant.source_field()) {
@@ -259,24 +280,24 @@ fn impl_enum(input: Enum) -> TokenStream {
                     let varsource = quote!(source);
                     let source_provide = if type_is_option(source_field.ty) {
                         quote_spanned! {source.span()=>
-                            if let std::option::Option::Some(source) = #varsource {
-                                source.provide(#demand);
+                            if let ::core::option::Option::Some(source) = #varsource {
+                                source.thiserror_provide(#request);
                             }
                         }
                     } else {
                         quote_spanned! {source.span()=>
-                            #varsource.provide(#demand);
+                            #varsource.thiserror_provide(#request);
                         }
                     };
                     let self_provide = if type_is_option(backtrace_field.ty) {
                         quote! {
-                            if let std::option::Option::Some(backtrace) = backtrace {
-                                #demand.provide_ref::<std::backtrace::Backtrace>(backtrace);
+                            if let ::core::option::Option::Some(backtrace) = backtrace {
+                                #request.provide_ref::<::thiserror::__private::Backtrace>(backtrace);
                             }
                         }
                     } else {
                         quote! {
-                            #demand.provide_ref::<std::backtrace::Backtrace>(backtrace);
+                            #request.provide_ref::<::thiserror::__private::Backtrace>(backtrace);
                         }
                     };
                     quote! {
@@ -285,8 +306,7 @@ fn impl_enum(input: Enum) -> TokenStream {
                             #source: #varsource,
                             ..
                         } => {
-                            #[allow(unused_imports)]
-                            use std::error::Error as _;
+                            use ::thiserror::__private::ThiserrorProvide as _;
                             #source_provide
                             #self_provide
                         }
@@ -299,19 +319,18 @@ fn impl_enum(input: Enum) -> TokenStream {
                     let varsource = quote!(source);
                     let source_provide = if type_is_option(source_field.ty) {
                         quote_spanned! {backtrace.span()=>
-                            if let std::option::Option::Some(source) = #varsource {
-                                source.provide(#demand);
+                            if let ::core::option::Option::Some(source) = #varsource {
+                                source.thiserror_provide(#request);
                             }
                         }
                     } else {
                         quote_spanned! {backtrace.span()=>
-                            #varsource.provide(#demand);
+                            #varsource.thiserror_provide(#request);
                         }
                     };
                     quote! {
                         #ty::#ident {#backtrace: #varsource, ..} => {
-                            #[allow(unused_imports)]
-                            use std::error::Error as _;
+                            use ::thiserror::__private::ThiserrorProvide as _;
                             #source_provide
                         }
                     }
@@ -320,13 +339,13 @@ fn impl_enum(input: Enum) -> TokenStream {
                     let backtrace = &backtrace_field.member;
                     let body = if type_is_option(backtrace_field.ty) {
                         quote! {
-                            if let std::option::Option::Some(backtrace) = backtrace {
-                                #demand.provide_ref::<std::backtrace::Backtrace>(backtrace);
+                            if let ::core::option::Option::Some(backtrace) = backtrace {
+                                #request.provide_ref::<::thiserror::__private::Backtrace>(backtrace);
                             }
                         }
                     } else {
                         quote! {
-                            #demand.provide_ref::<std::backtrace::Backtrace>(backtrace);
+                            #request.provide_ref::<::thiserror::__private::Backtrace>(backtrace);
                         }
                     };
                     quote! {
@@ -341,7 +360,7 @@ fn impl_enum(input: Enum) -> TokenStream {
             }
         });
         Some(quote! {
-            fn provide<'_demand>(&'_demand self, #demand: &mut std::any::Demand<'_demand>) {
+            fn provide<'_request>(&'_request self, #request: &mut ::core::error::Request<'_request>) {
                 #[allow(deprecated)]
                 match self {
                     #(#arms)*
@@ -354,19 +373,13 @@ fn impl_enum(input: Enum) -> TokenStream {
 
     let display_impl = if input.has_display() {
         let mut display_inferred_bounds = InferredBounds::new();
-        let use_as_display = if input.variants.iter().any(|v| {
+        let has_bonus_display = input.variants.iter().any(|v| {
             v.attrs
                 .display
                 .as_ref()
                 .map_or(false, |display| display.has_bonus_display)
-        }) {
-            Some(quote! {
-                #[allow(unused_imports)]
-                use thiserror::__private::{DisplayAsDisplay, PathAsDisplay};
-            })
-        } else {
-            None
-        };
+        });
+        let use_as_display = use_as_display(has_bonus_display);
         let void_deref = if input.variants.is_empty() {
             Some(quote!(*))
         } else {
@@ -374,19 +387,23 @@ fn impl_enum(input: Enum) -> TokenStream {
         };
         let arms = input.variants.iter().map(|variant| {
             let mut display_implied_bounds = Set::new();
-            let display = match &variant.attrs.display {
-                Some(display) => {
-                    display_implied_bounds = display.implied_bounds.clone();
-                    display.to_token_stream()
-                }
-                None => {
-                    let only_field = match &variant.fields[0].member {
-                        Member::Named(ident) => ident.clone(),
-                        Member::Unnamed(index) => format_ident!("_{}", index),
-                    };
-                    display_implied_bounds.insert((0, Trait::Display));
-                    quote!(std::fmt::Display::fmt(#only_field, __formatter))
-                }
+            let display = if let Some(display) = &variant.attrs.display {
+                display_implied_bounds.clone_from(&display.implied_bounds);
+                display.to_token_stream()
+            } else if let Some(fmt) = &variant.attrs.fmt {
+                let fmt_path = &fmt.path;
+                let vars = variant.fields.iter().map(|field| match &field.member {
+                    MemberUnraw::Named(ident) => ident.to_local(),
+                    MemberUnraw::Unnamed(index) => format_ident!("_{}", index),
+                });
+                quote!(#fmt_path(#(#vars,)* __formatter))
+            } else {
+                let only_field = match &variant.fields[0].member {
+                    MemberUnraw::Named(ident) => ident.to_local(),
+                    MemberUnraw::Unnamed(index) => format_ident!("_{}", index),
+                };
+                display_implied_bounds.insert((0, Trait::Display));
+                quote!(::core::fmt::Display::fmt(#only_field, __formatter))
             };
             for (field, bound) in display_implied_bounds {
                 let field = &variant.fields[field];
@@ -404,8 +421,9 @@ fn impl_enum(input: Enum) -> TokenStream {
         let display_where_clause = display_inferred_bounds.augment_where_clause(input.generics);
         Some(quote! {
             #[allow(unused_qualifications)]
-            impl #impl_generics std::fmt::Display for #ty #ty_generics #display_where_clause {
-                fn fmt(&self, __formatter: &mut std::fmt::Formatter) -> std::fmt::Result {
+            #[automatically_derived]
+            impl #impl_generics ::core::fmt::Display for #ty #ty_generics #display_where_clause {
+                fn fmt(&self, __formatter: &mut ::core::fmt::Formatter) -> ::core::fmt::Result {
                     #use_as_display
                     #[allow(unused_variables, deprecated, clippy::used_underscore_binding)]
                     match #void_deref self {
@@ -420,22 +438,34 @@ fn impl_enum(input: Enum) -> TokenStream {
 
     let from_impls = input.variants.iter().filter_map(|variant| {
         let from_field = variant.from_field()?;
+        let span = from_field.attrs.from.unwrap().span;
         let backtrace_field = variant.distinct_backtrace_field();
         let variant = &variant.ident;
         let from = unoptional_type(from_field.ty);
-        let body = from_initializer(from_field, backtrace_field);
-        Some(quote! {
-            #[allow(unused_qualifications)]
-            impl #impl_generics std::convert::From<#from> for #ty #ty_generics #where_clause {
-                #[allow(deprecated)]
-                fn from(source: #from) -> Self {
-                    #ty::#variant #body
-                }
+        let source_var = Ident::new("source", span);
+        let body = from_initializer(from_field, backtrace_field, &source_var);
+        let from_function = quote! {
+            fn from(#source_var: #from) -> Self {
+                #ty::#variant #body
             }
+        };
+        let from_impl = quote_spanned! {span=>
+            #[automatically_derived]
+            impl #impl_generics ::core::convert::From<#from> for #ty #ty_generics #where_clause {
+                #from_function
+            }
+        };
+        Some(quote! {
+            #[allow(
+                deprecated,
+                unused_qualifications,
+                clippy::elidable_lifetime_names,
+                clippy::needless_lifetimes,
+            )]
+            #from_impl
         })
     });
 
-    let error_trait = spanned_error_trait(input.original);
     if input.generics.type_params().next().is_some() {
         let self_token = <Token![Self]>::default();
         error_inferred_bounds.insert(self_token, Trait::Debug);
@@ -445,7 +475,8 @@ fn impl_enum(input: Enum) -> TokenStream {
 
     quote! {
         #[allow(unused_qualifications)]
-        impl #impl_generics #error_trait for #ty #ty_generics #error_where_clause {
+        #[automatically_derived]
+        impl #impl_generics ::thiserror::__private::Error for #ty #ty_generics #error_where_clause {
             #source_method
             #provide_method
         }
@@ -454,14 +485,22 @@ fn impl_enum(input: Enum) -> TokenStream {
     }
 }
 
+// Create an ident with which we can expand `impl Trait for #ident {}` on a
+// deprecated type without triggering deprecation warning on the generated impl.
+pub(crate) fn call_site_ident(ident: &Ident) -> Ident {
+    let mut ident = ident.clone();
+    ident.set_span(ident.span().resolved_at(Span::call_site()));
+    ident
+}
+
 fn fields_pat(fields: &[Field]) -> TokenStream {
     let mut members = fields.iter().map(|field| &field.member).peekable();
     match members.peek() {
-        Some(Member::Named(_)) => quote!({ #(#members),* }),
-        Some(Member::Unnamed(_)) => {
+        Some(MemberUnraw::Named(_)) => quote!({ #(#members),* }),
+        Some(MemberUnraw::Unnamed(_)) => {
             let vars = members.map(|member| match member {
-                Member::Unnamed(member) => format_ident!("_{}", member),
-                Member::Named(_) => unreachable!(),
+                MemberUnraw::Unnamed(index) => format_ident!("_{}", index),
+                MemberUnraw::Named(_) => unreachable!(),
             });
             quote!((#(#vars),*))
         }
@@ -469,22 +508,36 @@ fn fields_pat(fields: &[Field]) -> TokenStream {
     }
 }
 
-fn from_initializer(from_field: &Field, backtrace_field: Option<&Field>) -> TokenStream {
+fn use_as_display(needs_as_display: bool) -> Option<TokenStream> {
+    if needs_as_display {
+        Some(quote! {
+            use ::thiserror::__private::AsDisplay as _;
+        })
+    } else {
+        None
+    }
+}
+
+fn from_initializer(
+    from_field: &Field,
+    backtrace_field: Option<&Field>,
+    source_var: &Ident,
+) -> TokenStream {
     let from_member = &from_field.member;
     let some_source = if type_is_option(from_field.ty) {
-        quote!(std::option::Option::Some(source))
+        quote!(::core::option::Option::Some(#source_var))
     } else {
-        quote!(source)
+        quote!(#source_var)
     };
     let backtrace = backtrace_field.map(|backtrace_field| {
         let backtrace_member = &backtrace_field.member;
         if type_is_option(backtrace_field.ty) {
             quote! {
-                #backtrace_member: std::option::Option::Some(std::backtrace::Backtrace::capture()),
+                #backtrace_member: ::core::option::Option::Some(::thiserror::__private::Backtrace::capture()),
             }
         } else {
             quote! {
-                #backtrace_member: std::convert::From::from(std::backtrace::Backtrace::capture()),
+                #backtrace_member: ::core::convert::From::from(::thiserror::__private::Backtrace::capture()),
             }
         }
     });
@@ -528,22 +581,3 @@ fn type_parameter_of_option(ty: &Type) -> Option<&Type> {
         _ => None,
     }
 }
-
-fn spanned_error_trait(input: &DeriveInput) -> TokenStream {
-    let vis_span = match &input.vis {
-        Visibility::Public(vis) => Some(vis.pub_token.span()),
-        Visibility::Crate(vis) => Some(vis.crate_token.span()),
-        Visibility::Restricted(vis) => Some(vis.pub_token.span()),
-        Visibility::Inherited => None,
-    };
-    let data_span = match &input.data {
-        Data::Struct(data) => data.struct_token.span(),
-        Data::Enum(data) => data.enum_token.span(),
-        Data::Union(data) => data.union_token.span(),
-    };
-    let first_span = vis_span.unwrap_or(data_span);
-    let last_span = input.ident.span();
-    let path = quote_spanned!(first_span=> std::error::);
-    let error = quote_spanned!(last_span=> Error);
-    quote!(#path #error)
-}
